/*
 * $Id: rip.h,v 1.6 1998/09/04 20:35:11 masaki Exp $
 */

#ifndef _RIP_H
#define _RIP_H

#include <config.h>
#include <linked_list.h>
#include <trace.h>
#include <timer.h>
#include <schedule.h>
#include <hash.h>

#include <mrt.h>
#include <proto.h>
#include <interface.h>
#include <rib.h>

#define RIP_DEFAULT_PORT	520
#define RIP_MAX_PDU		512

#define RIP_REQUEST	1
#define RIP_RESPONSE	2

/* #define RIP_VERSION  1 */
#define RIP_VERSION     2       /* support RIP-2 only (no compatible mode) */

#undef RIP_POISONED_REVERSE	/* define if poisoned reverse */

#if 0				/* test */
#define RIP_TIMEOUT_INTERVAL	60
#define RIP_GARBAGE_INTERVAL	40
#define RIP_UPDATE_INTERVAL	15
#define RIP_UPDATE_JITTER 	0
#else
#define RIP_TIMEOUT_INTERVAL	180
#define RIP_GARBAGE_INTERVAL	120	/* after timeout */
#define RIP_UPDATE_INTERVAL	30
#define RIP_UPDATE_JITTER 	15
#endif
#define RIP_METRIC_INFINITY	16

/* These values come from gated */
#define RIP_FLASH_DELAY_MIN 1
#define RIP_FLASH_DELAY_MAX 5

typedef struct _rip_t {
    int proto;			/* PROTO_RIP or PROTO_RIPNG */
    u_long interface_mask;	/* mask of interfaces ripng is configed for */
    int sockfd;			/* socket we are listeing on */
    HASH_TABLE *hash;		/* hash of prefixes */
    trace_t *trace;

    rib_update_route_t update_call_fn;
    schedule_t *schedule;
    mtimer_t *timer;		/* timer used for sending update */
    mtimer_t *age;		/* aging routes */
    mtimer_t *flash;		/* flash update */

    pthread_mutex_t send_mutex_lock;
    pthread_mutex_t route_table_mutex_lock;
    pthread_mutex_t flash_update_mutex_lock;

    LINKED_LIST *ll_networks;
    LINKED_LIST *ll_interfaces;
    LINKED_LIST *ll_dlists;
    pthread_mutex_t mutex_lock;

    int flash_update_waiting;
    int num_routes;
} rip_t;

typedef struct _rip_attr_t {
    u_char type;
    prefix_t *nexthop;
    u_long ref_count;
    gateway_t *gateway;
    pthread_mutex_t mutex_lock;	/* lock down structure */
    u_long tag;

    u_char metric;
    u_long pref;
    time_t time;		/* time created */
} rip_attr_t;

typedef struct _rip_route_t {
    u_long flags;
    prefix_t *prefix;
    rip_attr_t *attr;
    rip_attr_t *imported;
    time_t time;		/* time received or updated */
    time_t delete;		/* time started deletion process */
} rip_route_t;

/* route flags */
#define RT_RIP_CHANGE	0x1
#define RT_RIP_DELETE	0x2

extern rip_t *RIP2;
extern rip_t *RIPNG;

/* user settable attributes */
enum RIP_ATTR {
    RIP_NULL = 0,
    RIP_TRACE_STRUCT,
    RIP_RT_UPDATE_FN,		/* function to call when ripng routes change */
    RIP_USE_PORTSERVER		/* use portserver library for listening */
};


/* public functions */

int rip_init (int proto, trace_t * trace);
void rip_start (int proto);
int rip_policy (rip_t * rip, prefix_t * prefix, rip_attr_t * attr,
		interface_t * out);
void rip_set (int proto, int first,...);
rip_attr_t *rip_new_attr (rip_t * rip, int metric);
rip_attr_t *rip_ref_attr (rip_attr_t * attr);
void rip_deref_attr (rip_attr_t * attr);
int rip_process_update (rip_t * rip, LINKED_LIST * ll_prefixes,
			LINKED_LIST * ll_attr);
void rip_schedule_receive_update (rip_t * rip);
int rip_update_route (prefix_t * prefix, generic_attr_t * new,
		      generic_attr_t * old, int pref);
int rip_show (int proto, uii_connection_t * uii);
int rip_show_routing_table (int proto, uii_connection_t * uii);

/* from rip2.c */
int rip2_init_listen (void);
void rip2_send_request (LINKED_LIST * ll_prefixes);
void rip2_send_routes (interface_t * interface, struct sockaddr_in *host,
		       int change);
void rip2_receive_update (void);
extern rip_t *RIP;

#ifdef HAVE_IPV6
#include <ripng.h>
#endif /* HAVE_IPV6 */

#endif /* _RIP_H */
